@@ -8,7 +8,7 @@
 
 """
 Notes:
-    Some tools for characterization
+    Some tools for characterization.
 """
 import inspect
 import multiprocessing
@@ -18,6 +18,7 @@ import random
 import re
 import time
 from collections.abc import Iterable
+from copy import deepcopy
 from functools import partial, wraps
 from itertools import chain
 from sys import getsizeof
@@ -65,8 +66,7 @@ def time_this_function(func):
 
 def check_random_state(seed):
     """
-    Turn seed into a random.RandomState instance.
-    if use the numpy random, please user the check_random_state in sklearn.
+    Turn seed into a random.RandomState instance. If use the numpy random, please user the check_random_state in sklearn.
 
     Parameters
     ----------
@@ -96,7 +96,6 @@ def parallelize(n_jobs, func, iterable, respective=False, tq=True, batch_size='a
                 parallel_para_dict=None, respective_kwargs=False,
                 **kwargs):
     """
-
     Parallelize the function for iterable.
 
     Examples
@@ -360,7 +359,6 @@ def parallelize_imap(n_jobs, func, iterable, tq=True):
 
 
 def parallelize_parameter(func, iterable, respective=False, **kwargs):
-    """Decrease your output size of your function"""
     import multiprocessing
     maxx = multiprocessing.cpu_count()
     n_jobs = maxx - 2
@@ -399,8 +397,8 @@ def parallelize_parameter(func, iterable, respective=False, **kwargs):
 
 def logg(func, printing=True, back=False):
     """
-    Get the name of function
-    use as a decorator
+    Get the name of function.
+    use as a decorator:@
 
     Parameters
     ----------
@@ -445,7 +443,7 @@ def logg(func, printing=True, back=False):
 
 def name_to_name(*iters, search, search_which=1, return_which=(1,), two_layer=False):
     """
-    search and rank the list.
+    Search and rank the list.
 
     Parameters
     ----------
@@ -570,11 +568,11 @@ class _TTClass(dict):
 
 class TTClass(_TTClass):
     """
-    quick time.
-    use tt object.
+    Quick time. Don't use this class use 'tt' object.
 
     Examples:
     -----------
+    >>> from mgetool.tool import tt
     >>> tt.t
     >>> a=4
     >>> tt.t
@@ -623,23 +621,6 @@ def def_pwd(path=None, change=False, verbose=False):
     return path
 
 
-def get_name_without_suffix(module_name):
-    """Get the name without suffix."""
-    if "-" in module_name:
-        print("'-' in '{}' is replaced by '_'".format(module_name))
-    module_name = module_name.replace("-", "_")
-
-    module_fc = re.compile(r"\W")
-    module_fc = module_fc.findall(module_name)
-    if module_fc[0] == "." and len(module_fc) == 1:
-        module_name = module_name.split(".")[0]
-    else:
-        module_fc.remove(".")
-        raise NameError("The string {} in module_name is special character.".format(module_fc))
-    print("Confirm the model name: {}".format(module_name))
-    return module_name
-
-
 def cmd_sys(d, cmd):
     """Run linux cmd"""
     old = os.getcwd()
@@ -659,6 +640,76 @@ def cmd_popen(d, cmd):
 
 tt = TTClass()
 
+
+def coarse_and_spilt_array(array: np.ndarray, tol: float = 0.5, method: str = None, n_cluster: int = 3) -> np.ndarray:
+    """
+    Split 1D ndarray by distance or group.
+
+    Args:
+        array: (np.ndarray) with shape (n,).
+        tol: (float) tolerance distance for spilt.
+        method:(str) default None. others: "agg", "k_means", "cluster", "k_means_user".
+        n_cluster: (int) number of cluster.
+
+    Returns:
+        labels: (np.ndarray) with shape (n,).
+
+    """
+    if method in ["agg", "k_means"]:
+        if method == "agg":
+            from sklearn.cluster import AgglomerativeClustering
+            ac = AgglomerativeClustering(n_clusters=None, distance_threshold=tol, compute_distances=True)
+        else:
+            from sklearn.cluster import KMeans
+            ac = KMeans(n_clusters=n_cluster)
+
+        ac.fit(array.reshape(-1, 1))
+        labels_ = ac.labels_
+        labels_max = np.max(labels_)
+        labels = deepcopy(labels_)
+        dis = np.array([np.mean(array[labels_ == i]) for i in range(labels_max + 1)])
+        dis_index = np.argsort(dis)
+        for i in range(labels_max + 1):
+            labels[labels_ == i] = dis_index[i]
+        return labels
+    else:
+        # use tol directly
+        array = array.ravel()
+        array_sindex = np.argsort(array)
+        array_sort = array[array_sindex]
+        i = 0
+        label = 0
+        labels = []
+        while i < len(array_sort):
+            if i == 0:
+                labels.append(label)
+            else:
+                if array_sort[i] - array_sort[i - 1] < tol:
+                    labels.append(label)
+                else:
+                    label += 1
+                    labels.append(label)
+            i += 1
+
+        labels = np.array(labels)[np.argsort(array_sindex)]
+
+        return labels
+
+
+def dos2unix(file, out_file=None):
+    """
+
+    Args:
+        file: (str,) input file name
+        out_file: (str,) input file name, if None, cover the input.
+    """
+    with open(file, 'rb') as infile:
+        content = infile.read()
+    out_file = file if out_file is None else out_file
+    with open(out_file, 'wb') as output:
+        for line in content.splitlines():
+            output.write(line + b'\n')
+
 if __name__ == "__main__":
     # def func(n, _=None):
     #     # time.sleep(0.0001)
